//-------------------------------------------
//UART0.c : source file for UART0
//-------------------------------------------

//-------------------------------------------
// Definition of local includes
//-------------------------------------------

#include <avr/interrupt.h>

#include "UART.h"

#ifdef UART0

#ifdef PRINTF
  #include <stdint.h>
  #include <avr/io.h>
  int  UART0_putchar(char c, FILE *stream);    // used for printf()
  FILE uart_str = FDEV_SETUP_STREAM(UART0_putchar, NULL, _FDEV_SETUP_RW);  //setup printf
#endif // #ifdef PRINTF

//-------------------------------------------
// Definition of local constants
//-------------------------------------------

// Buffersizes must be 2^n
#define T0BUFSIZE  256
#define R0BUFSIZE  256
#define T0MASK     (T0BUFSIZE-1)
#define R0MASK     (R0BUFSIZE-1)
#define R0BUFLEN   (u08)(r0_in - r0_out)
#define T0BUFLEN   (u08)(t0_in - t0_out)

//-------------------------------------------
// Declaration of local variables
//-------------------------------------------

volatile u08 t0buf[T0BUFSIZE];
volatile u08 r0buf[R0BUFSIZE];
volatile u08 r0msg =0;
volatile u08 t0_in =0;
volatile u08 t0_out=0;
volatile u08 r0_in =0;
volatile u08 r0_out=0;
volatile u08 echo0=0;


//-------------------------------------------
// Declaration of local functions
//-------------------------------------------

//-------------------------------------------
// Code for functions
//-------------------------------------------

ISR(USART0_RX_vect)
//-------------------------------------------------------
// RX-ISR for UART0
//-------------------------------------------------------
{
  u08 c = UDR0;	                    // Get received char
  u08 echo = echo0;
  if (echo)
  {
    UART0_putc(c);                  // echo
  }
  if (c==0x0D)                      // if char = [CR]
  {
    r0buf[r0_in++ & R0MASK] = 0;    // add stringend to buffer
    if (echo)
    {
      UART0_putc(0x0A);             // append line feed to echo
    }
    r0msg++;                        // increment message counter
  }
  if (c==0x08)                      // if backspace
  {
    if (r0buf[(r0_in-1) & R0MASK])  // if there is a char, left from cursor
    {
      r0buf[(--r0_in) & R0MASK]=0;  // then delete it  and go one char back.
      if (echo)
      {
        UART0_putc(' ');            // echo SPACE to delete char on Terminal
        UART0_putc(0x08);           // Back-SPACE again
      }
    }
  }
  if (c > 0x19)                     // if printable char
  {
    r0buf [r0_in++ & R0MASK] = c;   // add char to buffer
  }
}

ISR(USART0_UDRE_vect)
//-------------------------------------------------------
// TX-ISR for UART0
// Data register empty indicating that
// next char can be transmitted
//-------------------------------------------------------
{
  if (t0_in != t0_out)               // If TX-Buffer contains Data
  {
    UDR0 = t0buf [t0_out & T0MASK];  // then send next char
    t0_out++;	                     // pointer to next char
  }
  else
  {
    UCSR0B &= ~(1<<UDRIE0);          // else disable the DataRegisterEmpty Interrupt
  }
}

void UART0_init(u16 baud)
//-------------------------------------------------------
// init UAR0
//-------------------------------------------------------
{
  UBRR0  = baud;                              // set Baudrate
  UCSR0A = (1<<U2X0);                         // enable double speed mode
  UCSR0C = (3<<UCSZ00);                       // Async; 8 Data; No Par; 1 Stop
  UCSR0B = (1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0); // Rx-Interrupt; RX-Enable; TX-Enable
  //-----------------------------------------------------------------------------
  // initialize stout for printf()
  //-----------------------------------------------------------------------------
#ifdef PRINTF
  stdout = &uart_str;
  stdin  = NULL;
  stderr = NULL;
#endif // #ifdef PRINTF
  //-----------------------------------------------------------------------------
}

void UART0_echo(u08 echo)
{
  echo0=echo;
}

u08 UART0_getl(char *str)
//-------------------------------------------------------
// Check if a CR-terminated line is received on UART0
// if 'YES'
// then copy line to *str and return(StringLength)
// else return(StringLength=0)
//-------------------------------------------------------
{
  u08 i=0;
  if(r0msg)                               // if message in buffer available
  {
    r0msg--;                              // decrease message buffer
    str[i]=r0buf [r0_out++ & R0MASK];     // get first char
    while(str[i])                         // while not stringend
    {
      str[++i]=r0buf [r0_out++ & R0MASK]; // get next char
    }
    if (i==0)
    {
      str[0]=0x0D;                        // insert CR
      str[1]=0x00;                        // and insert stringend
      i=1;                                // adjust string length
    }
  }
  return(i);                              // return(StringLength)
}

void UART0_putc(char c)
//-------------------------------------------------------
// Put a char to the transmit buffer.
// Wait if buffer is full, else add data
// to the transmit buffer and enable UDRIE0
//-------------------------------------------------------
{
  while(T0BUFLEN > (T0BUFSIZE-5));      // wait if buffer full
  t0buf [t0_in++ & T0MASK] = c;         // next char
  UCSR0B |= (1<<UDRIE0);	        // enable UDR0 empty interrupt
}

#ifdef PRINTF
int UART0_putchar(char c, FILE *stream)
//-------------------------------------------------------
// used for printf()
// Put a char to the transmit buffer.
// Wait if buffer is full, else add data
// to the transmit buffer and enable UDRIE0
//-------------------------------------------------------
{
  while(T0BUFLEN > (T0BUFSIZE-3));      // wait if buffer full
  if (c == '\n')
  {
    t0buf [t0_in++ & T0MASK] = '\r';    // add [CR] before [NL]
  }
  t0buf [t0_in++ & T0MASK] = c;         // next char
  UCSR0B |= (1<<UDRIE0);                // enable UDR0 empty interrupt
  return 0;
}
#endif  //#ifdef PRINTF

void UART0_puts(char *str)
//---------------------------------------------------
// send a string to UART0 Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------
{
  u08 i=0;
  while (str[i])
  {
    UART0_putc(str[i++]);
  }
}

#ifdef FARROM

void UART0_puts_PF(u32 str)
//---------------------------------------------------
// send a ProgMem string to UART0 Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------
{
  u08 c=pgm_read_byte_far(str);
  while (c)
  {
    UART0_putc(c);
    c=pgm_read_byte_far(++str);
  }
}

void UART0_putl_PF(u32 str)
//---------------------------------------------------
// send a ProgMem string to UART0 Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------
{
  u08 c=pgm_read_byte_far(str);
  while (c)
  {
    UART0_putc(c);
    c=pgm_read_byte_far(++str);
  }
  UART0_putc('\r');
  UART0_putc('\n');
}

#else

void UART0_puts_P(PGM_P str)
//---------------------------------------------------
// send a ProgMem string to UART0 Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------
{
  char c=pgm_read_byte(str);
  while (c)
  {
    UART0_putc(c);
    c=pgm_read_byte(++str);
  }
}

void UART0_putl_P(PGM_P str)
//---------------------------------------------------------
// send a ProgMem string + CR + LF to UART Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------------
{
  char c=pgm_read_byte(str);
  while (c)
  {
    UART0_putc(c);
    c=pgm_read_byte(++str);
  }
  UART0_putc('\r');
  UART0_putc('\n');
}

#endif //#ifdef FARROM

//=========================================================

#ifdef TRACE

//--------------------------------------------------------
// put timestamp + string to TX-Buffer
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART0_traces(char* str)
{
  UART0_puts(TIMER_get());
  UART0_puts(str);
}

//--------------------------------------------------------
// put timestamp + string + [CR+LF] to TX-Buffer
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART0_tracel(char* str)
{
  UART0_puts(TIMER_get());
  UART0_puts(str);
  UART0_putc('\r');
  UART0_putc('\n');
}

//--------------------------------------------------------
// put '------------------------------------' + [CR+LF]
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART0_tracem(void)
{
  UART0_puts_P(PSTR("----------------------------------------------------------------------\r\n"));
}

//--------------------------------------------------------
// put timestamp + pgmemstring  to TX-Buffer
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART0_traces_P(PGM_P str)
{
  UART0_puts(TIMER_get());
  UART0_puts_P(str);
}

//--------------------------------------------------------
// put timestamp + pgmemstring + [CR+LF] to TX-Buffer
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART0_tracel_P(PGM_P str)
{
  UART0_puts(TIMER_get());
  UART0_puts_P(str);
  UART0_putc('\r');
  UART0_putc('\n');
}

#endif //#ifdef TRACE
#endif //#ifdef "UART0
