//-------------------------------------------
//UART0.c : source file for UART1
//-------------------------------------------

//-------------------------------------------
// Definition of local includes
//-------------------------------------------

//#include "defines.h"

#include <avr/interrupt.h>

#include "UART.h"

#ifdef UART1

#ifdef PRINTF
  #include <stdint.h>
  #include <avr/io.h>
  int  UART1_putchar(char c, FILE *stream);  // used for printf()
  FILE uart_str = FDEV_SETUP_STREAM(UART1_putchar, NULL, _FDEV_SETUP_RW);  //setup printf
#endif // #ifdef PRINTF

//-------------------------------------------
// Definition of local constants
//-------------------------------------------

// Buffersizes must be 2^n
#define T1BUFSIZE  256
#define R1BUFSIZE  256
#define T1MASK     (T1BUFSIZE-1)
#define R1MASK     (R1BUFSIZE-1)
#define R1BUFLEN   (u08)(r1_in - r1_out)
#define T1BUFLEN   (u08)(t1_in - t1_out)

//-------------------------------------------
// Declaration of local variables
//-------------------------------------------

volatile u08 t1buf[T1BUFSIZE];
volatile u08 r1buf[R1BUFSIZE];
volatile u08 r1msg =0;
volatile u08 t1_in =0;
volatile u08 t1_out=0;
volatile u08 r1_in =0;
volatile u08 r1_out=0;
volatile u08 echo1=0;

//-------------------------------------------
// Declaration of local functions
//-------------------------------------------

//-------------------------------------------
// Code for functions
//-------------------------------------------

ISR(USART1_RX_vect)
//-------------------------------------------------------
// RX-ISR for UART1
//-------------------------------------------------------
{
  u08 c = UDR1;	   	            // Get received char
  u08 echo = echo1;
  if (echo)
  {
    UART1_putc(c);                  // echo
  }
  if (c==0x0D)                      // if char = [CR]
  {
    r1buf[r1_in++ & R1MASK] = 0;    // add stringend to buffer
    if (echo)
    {
      UART1_putc(0x0A);             // append line feed to echo
    }
    r1msg++;                        // increment message counter
  }
  if (c==0x08)                      // if backspace
  {
    if (r1buf[(r1_in-1) & R1MASK])  // if there is a char, left from cursor
    {
      r1buf[(--r1_in) & R1MASK]=0;  // then delete it  and go one char back.
      if (echo)
      {
       UART1_putc(' ');             // echo SPACE to delete char on Terminal
       UART1_putc(0x08);            // Back-SPACE again
      }
    }
  }
  if (c > 0x19)                     // if printable char
  {
    r1buf [r1_in++ & R1MASK] = c;   // add char to buffer
  }
}

ISR(USART1_UDRE_vect)
//-------------------------------------------------------
// TX-ISR for UART1
// Data register empty indicating that
// next char can be transmitted
//-------------------------------------------------------
{
  if (t1_in != t1_out)               // If TX-Buffer contains Data
  {
    UDR1 = t1buf [t1_out & T1MASK];  // then send next char
    t1_out++;	                     // pointer to next char
  }
  else
  {
    UCSR1B &= ~(1<<UDRIE1);          // else disable the DataRegisterEmpty Interrupt
  }
}

void UART1_init(u16 baud)
//-------------------------------------------------------
// init UART1
//-------------------------------------------------------
{
  UBRR1 = baud;                               // set Baudrate
  //UBRR1H = 0x00;                            // only used for < 9600 Baud
  UCSR1A = (1<<U2X1);                         // enable double speed mode
  UCSR1C = (1<<UCSZ11)|(1<<UCSZ10);           // Async; 8 Data; No Par; 1 Stop
  UCSR1B = (1<<RXCIE1)|(1<<RXEN1)|(1<<TXEN1); // Rx-Interrupt; RX-Enable; TX-Enable
  //-----------------------------------------------------------------------------
  // initialize stout for printf()
  //-----------------------------------------------------------------------------
#ifdef PRINTF
  stdout = &uart_str;
  stdin  = NULL;
  stderr = NULL;
#endif // #ifdef PRINTF
  //-----------------------------------------------------------------------------
}

void UART1_echo(u08 echo)
{
  echo1=echo;
}

u08 UART1_getl(char *str)
//-------------------------------------------------------
// Check if a CR-terminated line is received on UART0
// if 'YES'
// then copy line to *str and return(StringLength)
// else return(StringLength=0)
//-------------------------------------------------------
{
  u08 i=0;
  if(r1msg)                               // if message in buffer available
  {
    r1msg--;                              // decrease message buffer
    str[i]=r1buf [r1_out++ & R1MASK];     // get first char
    while(str[i])                         // while not stringend
    {
      str[++i]=r1buf [r1_out++ & R1MASK]; // get next char
    }
    if (i==0)
    {
      str[0]=0x0D;                        // insert CR
      str[1]=0x00;                        // and insert stringend
      i=1;                                // adjust string length
    }
  }
  return(i);                              // return(StringLength)
}

void UART1_putc(char c)
//-------------------------------------------------------
// Put a char to the transmit buffer.
// Wait if buffer is full, else add data
// to the transmit buffer and enable UDRIE1
//-------------------------------------------------------
{
  while(T1BUFLEN > (T1BUFSIZE-5));    // wait if buffer full
  t1buf[t1_in++ & T1MASK] = c;        // next char
  UCSR1B |= (1<<UDRIE1);	      // enable UDR1 empty interrupt
}

#ifdef PRINTF
int UART1_putchar(char c, FILE *stream)
//-------------------------------------------------------
// used for printf()
// Put a char to the transmit buffer.
// Wait if buffer is full, else add data
// to the transmit buffer and enable UDRIE1
//-------------------------------------------------------
{
  while(T1BUFLEN > (T1BUFSIZE-3));      // wait if buffer full
  if (c == '\n')
  {
    t1buf [t1_in++ & T1MASK] = '\r';    // add [CR] before [NL]
  }
  t1buf [t1_in++ & T1MASK] = c;         // next char
  UCSR1B |= (1<<UDRIE1);                // enable UDR0 empty interrupt
  return 0;
}
#endif  //#ifdef PRINTF

void UART1_puts(char *str)
//---------------------------------------------------
// send a string to UART0 Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------
{
  u08 i=0;
  while (str[i])
  {
    UART1_putc(str[i++]);
  }
}

#ifdef FARROM

void UART1_puts_PF(u32 str)
//---------------------------------------------------
// send a ProgMem string to UART0 Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------
{
  u08 c=pgm_read_byte_far(str);
  while (c)
  {
    UART1_putc(c);
    c=pgm_read_byte_far(++str);
  }
}

void UART1_putl_PF(u32 str)
//---------------------------------------------------
// send a ProgMem string to UART0 Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------
{
  u08 c=pgm_read_byte_far(str);
  while (c)
  {
    UART1_putc(c);
    c=pgm_read_byte_far(++str);
  }
  UART1_putc('\r');
  UART1_putc('\n');
}

#else
void UART1_puts_P(PGM_P str)
//---------------------------------------------------
// send a ProgMem string to UART0 Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------
{
  char c=pgm_read_byte(str);
  while (c)
  {
    UART1_putc(c);
    c=pgm_read_byte(++str);
  }
}

void UART1_putl_P(PGM_P str)
//---------------------------------------------------------
// send a ProgMem string + CR + LF to UART Transmit-buffer
// wait, if the transmit buffer is full
//---------------------------------------------------------
{
  char c=pgm_read_byte(str);
  while (c)
  {
    UART1_putc(c);
    c=pgm_read_byte(++str);
  }
  UART1_putc('\r');
  UART1_putc('\n');
}
#endif //#ifdef FARROM


//=========================================================

#ifdef TRACE

//--------------------------------------------------------
// put timestamp + string to TX-Buffer
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART1_traces(char* str)
{
  UART1_puts(TIMER_get());
  UART1_puts(str);
}

//--------------------------------------------------------
// put timestamp + string + [CR+LF] to TX-Buffer
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART1_tracel(char* str)
{
  UART1_puts(TIMER_get());
  UART1_puts(str);
  UART1_putc('\r');
  UART1_putc('\n');
}

//--------------------------------------------------------
// put '------------------------------------' + [CR+LF]
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART1_tracem(void)
{
  UART1_puts_P(PSTR("----------------------------------------------------------------------\r\n"));
}

//--------------------------------------------------------
// put timestamp + pgmemstring  to TX-Buffer
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART1_traces_P(PGM_P str)
{
  UART1_puts(TIMER_get());
  UART1_puts_P(str);
}

//--------------------------------------------------------
// put timestamp + pgmemstring + [CR+LF] to TX-Buffer
// wait, if the transmit buffer is full
//--------------------------------------------------------
void UART1_tracel_P(PGM_P str)
{
  UART1_puts(TIMER_get());
  UART1_puts_P(str);
  UART1_putc('\r');
  UART1_putc('\n');
}

#endif //#ifdef TRACE
#endif //#ifdef UART1
