//------------------------------------------------------------------------------
// boot.c : Source file for the bootloader
//------------------------------------------------------------------------------

/*--- INCLUDE FILES ----------------------------------------------------------*/

#include <util/delay.h>

#include "main.h"
#include "command.h"
#include "UART.h"
#include "boot.h"

//------------------------------------------------------------------------------
// Chip Definitions for AtMega???
//------------------------------------------------------------------------------

#define _B4096    // Select Bootsector Size

#define sig_byte3 0x1E
#define sig_byte2 0x96
#define sig_byte1 0x0A

#ifdef _B512
  #define APP_PAGES ((2*65536UL / SPM_PAGESIZE)- (2*512 / SPM_PAGESIZE ))
  #define APP_END APP_PAGES * SPM_PAGESIZE
#endif
#ifdef _B1024
  #define APP_PAGES ((2*65536UL / SPM_PAGESIZE)- (2*1024 / SPM_PAGESIZE ))
  #define APP_END APP_PAGES * SPM_PAGESIZE
#endif
#ifdef _B2048
  #define APP_PAGES ((2*65536UL / SPM_PAGESIZE)- (2*2048 / SPM_PAGESIZE ))
  #define APP_END APP_PAGES * SPM_PAGESIZE
#endif
#ifdef _B4096
  #define APP_PAGES ((2*65536UL / SPM_PAGESIZE)- (2*4096 / SPM_PAGESIZE ))
  #define APP_END APP_PAGES * SPM_PAGESIZE
#endif

#define PG_LO  ((u16)((SPM_PAGESIZE<<0)-1))
#define PG_HI  ((u16)((SPM_PAGESIZE<<1)-1))
#define PG_SEL ((u16)(PG_HI & ~PG_LO))

//------------------------------------------------------------------------------
// GLOBAL VARIABLES
//------------------------------------------------------------------------------
u08 page[SPM_PAGESIZE<<1];   // create temp buffer for two pages
u08 pg_new, pg_old;          // next and last page
u16 pg_addr;

// address of data to program

union {u32 laddr; u16 saddr[2]; u08 baddr[4];} mem;  // Flash-Address


//------------------------------------------------------------------------------
// FUNCTION PROTOTYPES
//------------------------------------------------------------------------------

void empty_buf(u08 pg_sel);
void store_page(u08 pg_sel);

//------------------------------------------------------------------------------
// IMPLEMENTATION
//------------------------------------------------------------------------------


void flash_erase(void)
//------------------------------------------------
// Erase the main flash excepting BLS
//------------------------------------------------
{
  mem.laddr=0;
  UART_putl_I("Erasing Sectors");
  while (APP_END > mem.laddr)
  {
    wdt_reset();
    boot_spm_busy_wait();              // wait until last SPM operation is performed
    cli();
    boot_page_erase(mem.laddr);        // Perform page erase
    sei();
    UART_putc('.');
    mem.laddr += SPM_PAGESIZE;
  }
  empty_buf(0x01);                     // fill page1 buffer with 0xFF
  empty_buf(0x02);                     // fill page2 buffer with 0xFF
  //----------------------------------------------------------------------------------------
  _delay_ms(10);                                   // wait until "done" is sent
  UPDATE = UPDATE_NOAPP;                           // set flag for no application available
  UPDATE |= UPDATE_READY;                          // and set update ready flag
  //----------------------------------------------------------------------------------------
  mem.laddr=0;
  pg_old=0;
  pg_new=0;
}


u08 process_record(void)
//------------------------------------------------
// process and programm an intel hex record line
//------------------------------------------------
{
  u08	i, cnt, type, temp=0;
  //------------------------------------------------------------------------------
  temp = hex2bin(MSG+1,RSP,0);             // convert hex record to binary array (=> checksum)
  if (temp==0)                             // if checksum is ok
  {
    cnt  = RSP[0]+4;                       // calculate total record length
    type = RSP[3];                         // get record type
    if (type==0x00)                        // if data record
    {
      mem.baddr[1] = RSP[1];               // get address high
      mem.baddr[0] = RSP[2];               // get address low
      pg_new = 0x01;                       // set page select for low page
      if (mem.saddr[0] & PG_SEL)           // check page select
      {
        pg_new = 0x02;                     // set page select for high page
      }
      if (pg_new != pg_old)                // if next page
      {
        store_page(pg_old);                // then store previous page
        pg_old = pg_new;                   // set old page select to new page select
        pg_addr=(mem.laddr & ~(u32)PG_LO); // get base address for new page
      }
      for (i=4; i<cnt; i++)                // for each data byte of current record
      {
        page[mem.saddr[0] & PG_HI]=RSP[i]; // store data byte to data page
        mem.saddr[0]++;                    // increment mem address
      }
    }
    if (type==0x01)                        // if end-of-file record
    {
      pg_new = 0x01;                       // set page select for low page
      if (mem.saddr[0] & PG_SEL)           // check page select
      {
        pg_new = 0x02;                     // set page select for high page
      }
      if (pg_new != pg_old)                // if page has changed
      {
        store_page(pg_old);                // then store previous page
      }
      store_page(pg_new);                  // store actual page
      UPDATE &= ~UPDATE_READY;             // clear programming ready flag
      boot_spm_busy_wait();
      cli();
      boot_rww_enable();	           // Re-enable the RWW section
      sei();
      UART_putl_I("\r\nEnd Of File");
      UART_echo(ON);
      UPDATE &= UPDATE_ERROR;              // mask programming flag (keep ERROR)
      if(UPDATE!=UPDATE_DONE)              // if there was an error
      {
        temp=0xFF;                          // then show "failed" on terminal
      }
    }
    if (type==0x02)                        // if segment address record
    {
      mem.baddr[2] = RSP[4]>>4;            // add segment address (0x00..0x0F)
    }
  }
  return(temp);                            // return checksum
}


void store_page(u08 pg_sel)
{
  u16 data,start,stop;
  if (pg_sel)
  {
    if (pg_sel==0x01)
    {
      start=0;
      stop=SPM_PAGESIZE;
    }
    else
    {
      start=SPM_PAGESIZE;
      stop=(SPM_PAGESIZE<<1);
    }
    //----------------------------------------------
    // store data in temporary flash buffer
    //----------------------------------------------
    boot_spm_busy_wait();                  // wait until last SPM operation is performed
    for (u16 i=start; i<stop; i += 2)
    {
      data = page[i];                      // get low byte
      page[i]=0xFF;                        // clear low byte
      data |= (page[i+1] << 8);            // get high byte (do byte-swap)
      page[i+1]=0xFF;                      // clear high byte
      cli();
      boot_page_fill(pg_addr,data);        // call asm routine to load temporary flash buffer
      sei();
      pg_addr += 2;                        // select next word in temporary buffer
    }
    //----------------------------------------------
    // write page to the flash
    //----------------------------------------------
    cli();
    boot_page_write((pg_addr-2) & ~(u16)PG_LO);
    sei();
  }
}


void empty_buf(u08 pg_sel)
{
  if (pg_sel==0x01)
  {
    for (u16 i=0; i<SPM_PAGESIZE; i +=2)
    {
       page[i]=0xFF;
    }
  }
  if (pg_sel==0x02)
  {
    for (u16 i=SPM_PAGESIZE; i<(SPM_PAGESIZE<<1); i++)
    {
       page[i]=0xFF;
    }
  }
}

