//--------------------------------------------------------
// Command.c : source file for Commands
//--------------------------------------------------------

//-------------------------------------------
// Definition of local includes
//-------------------------------------------

//#include <avr/eeprom.h>
#include <util/delay.h>

#include "main.h"
#include "command.h"
#include "UART.h"
#include "boot.h"

//-------------------------------------------
// Definition of local constants
//-------------------------------------------

#define NACK 0x00
#define DONE 0x01
#define SACK 0x02
#define FBRG (u32)(F_CPU/8-4800)

//-------------------------------------------
// Declaration of local variables
//-------------------------------------------

u08 cmd = NACK;

//-------------------------------------------
// Declaration of local functions
//-------------------------------------------
void cmd_JA(void);   // jump to Application at Address 0x0000
void cmd_JB(void);   // jump to Bootlader at Address 0xF000
void cmd_RE(void);   // watchdog reset (Go Start)

void cmd_PS(void);   // initialize a firmware upgrade

void cmd_BR(void);   // set baudrate
void cmd_HE(void);   // Help

void hex_rec(void);  // if Intel HEX-record command => hex_rec()

//-------------------------------------------
// Code for functions
//-------------------------------------------

void cmd_check()
//-------------------------------------------
// Commando-Interpreter for any command
// wich is received on UART0 or UART1
//-------------------------------------------
{
  u08 msg0, msg1;
  if (LEN > 1)                        // command length > 1 char
  {
    cmd = NACK;                       // preset cmd flag
    msg0=MSG[0];
    msg1=MSG[1];
    if (msg0==':')
    {
      hex_rec();                      // if Intel HEX-record command => hex_rec()
    }
    else
    {
      msg0 = (msg0 | 0x20);           // convert to lower case
      msg1 = (msg1 | 0x20);           // convert to lower case
      //--------------------------------------------------------------------------
      if (msg0=='b')
      {
        if (msg1=='r') cmd_BR();      // jump to set Baudrate
      }
      //--------------------------------------------------------------------------
      if (msg0=='h')
      {
        cmd_HE();                      // Help
      }
      //--------------------------------------------------------------------------
      if (msg0=='j')
      {
        if (msg1=='a') cmd_JA();        // jump to Application
        if (msg1=='b') cmd_JB();        // jump to Bootlader
      }
      if (msg0=='r')
      {
        if (msg1=='e') cmd_RE();        // reset command forces a watchdog reset
      }
      //--------------------------------------------------------------------------
      if (msg0=='p')                    // if Programm command
      {
        if (msg1=='s') cmd_PS();        // initialize a firmware upgrade
      }
      //--------------------------------------------------------------------------
    }
  }
  else
  {
    cmd = DONE;                       // preset cmd flag
  }
  if (cmd==SACK) put_ack('A');        // send ACK
  if (cmd==NACK) put_ack('N');        // send NACK
}

void hex_rec(void)
{
  if (UPDATE & UPDATE_READY)           // If programming mode was entered
  {
    if (LEN > 10)                      // check for valid parameter length
    {
      if (process_record())            // process record. If an error occur
      {
        UPDATE |= UPDATE_ERROR;        // then set error bit; => NACK
      }
      else
      {
        cmd=DONE;                      // command is done, no further response
      }
    }
  }
}


void cmd_HE(void)
//---------------------------------------
// Display help screen
//---------------------------------------
{
  UART_echo(OFF);
  UART_putl_I("---------------------------------------------------------------");
  UART_putl_I("=> Bootloader Commands:");
  UART_putl_I("---------------------------------------------------------------");
  UART_putl_I(">HELP       // Display this help-screen");
  UART_putl_I(">BR <br>    // Set UART Baudrate to <br> ");
  UART_putl_I(">JAPP       // Jump to Application");
  UART_putl_I(">JBL        // Jump to Bootloader");
  UART_putl_I(">RESET      // SW-Restart");
  UART_putl_I("---------------------------------------------------------------");
  UART_putl_I(">PS         // Programing-Start: Init download and erase FLASH");
  UART_putl_I(">PE         // Programing-End:   Finalize the download");
  UART_putl_I("---------------------------------------------------------------");
  UART_echo(ON);
  _delay_ms(50);
  cmd=DONE;                      // command is done, no further response
}

void cmd_PS(void)
//---------------------------------------
// initialize a firmware upgrade
//---------------------------------------
{
  flash_erase();                                 // erase the flash
  cmd=SACK;                                      // command is done, send acknowledge
  UART_echo(OFF);
}

void cmd_BR(void)
//---------------------------------------
// Set the Baudrate of the UART
//---------------------------------------
{
  u32 baudrate;
  u16 baud;
  u08 i=0;
  for (i=0; i<=LEN; i++)
  {
    if ((MSG[i] >= '0') && (MSG[i] <= '9')) break;
  }
  baudrate = atol((char*)&MSG[i]);
  baud = check_baudrate(&baudrate);
  if (baud)
  {
    eeprom_write_block(&baudrate, EEPTR_BAUD, sizeof(baudrate));
    UART_puts_I("New Baudrate = ");
    UART_puts((char*)&MSG[i]);
    UART_puts("\r\n");
    _delay_ms(50);                 // wait until message is sent
    cli();                         // disable interrupt
    UART_init(baud);               // init UART
    sei();                         // enable interrupt
  }
  else
  {
    UART_putc('<');
    UART_puts((char*)&MSG[i]);
    UART_putc('>');
    UART_putl_I(" Invalid Baudrate");
  }
  cmd=DONE;  // and send acknowledge
}

void cmd_RE(void)
{
  wdt_reset();                // perform a last watchdog reset
  wdt_enable(WDTO_30MS);      // enable watchdog reset after 30ms
  while(1);                   // wait for watchdog reset
}

void cmd_JA(void)
{
  //-------------------------------------------------------------------
  cli();                      // disable interrups
  MCUCR =  (1<<IVCE);         // enable interruptvectors change
  MCUCR =  (0<<IVSEL);        // move interruptvectors to the Application-Sector
  //-------------------------------------------------------------------
  asm("jmp 0x0000");        // Jump to application sector
}

void cmd_JB(void)
{
  //-------------------------------------------------------------------
  UART_putl_I(" ");
  _delay_ms(20);
  cli();                      // disable interrups
  MCUCR =  (1<<IVCE);         // enable interruptvectors change
  MCUCR =  (1<<IVSEL);        // move interruptvectors to the Bootloader-Sector
  //-------------------------------------------------------------------
  asm("jmp 0xE000");          // Jump to bootloader. Attention: ==> "jmp 0x7000"
}

void put_rsp(u08 *str)
{
  UART_puts((char*)str);
  UART_putc('\r');
  UART_putc('\n');
}

void put_ack(u08 ack)
{
  if (ack=='A')
  {
    UART_putl_I("Ok");
  }
  else
  {
    UART_puts_I("Error <");
    UART_puts((char*)MSG);
    UART_putl_I(">");
  }
}

u16 check_baudrate(u32 *baudrate)
{
  u16 retval=0;
  if (*baudrate ==  9600) retval=BR_9600;
  if (*baudrate == 19200) retval=BR_19200;
  if (*baudrate == 38400) retval=BR_38400;
  if (*baudrate == 57600) retval=BR_57600;
  if (*baudrate ==115200) retval=BR_115200;
  return(retval);
}


